/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package mpi.dcr.ISO12620;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.URL;
import java.util.logging.Logger;

import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * <b>NameSection</b> is generated from ISO12620_revised.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element java:className="NameSection" name="struct">
 *       <attribute name="type">
 *         <value>NS</value>
 *       </attribute>
 *       <ref name="attId"/>
 *       <ref java:propertyName="name" name="name_struct_NS"/>
 *       <ref java:propertyName="status" name="status_struct_NS"/>
 * 		 <zeroOrMore>
 *         <ref java:propertyName="notes" name="note_struct_NS"/>
 *       </zeroOrMore>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element java:className="NameSection" name="struct"&gt;
 *       &lt;attribute name="type"&gt;
 *         &lt;value&gt;NS&lt;/value&gt;
 *       &lt;/attribute&gt;
 *       &lt;ref name="attId"/&gt;
 *       &lt;ref java:propertyName="name" name="name_struct_NS"/&gt;
 *       &lt;ref java:propertyName="status" name="status_struct_NS"/&gt;
 *       &lt;zeroOrMore&gt;
 *         &lt;ref java:propertyName="notes" name="note_struct_NS"/&gt;
 *       &lt;/zeroOrMore&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version ISO12620_revised.rng (Fri Sep 30 12:49:34 CEST 2005)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class NameSection implements java.io.Serializable, Cloneable {
    /** Logger for this class and subclasses */
    protected final static Logger logger = Logger.getLogger("DCR");

    /**
	 * serialVersionID(generated)
	 */
	private static final long serialVersionUID = -7585605491780617383L;

	public static final String TYPE_NS = "NS";

    private String type_;
    private String id_;
    private Name name_;
    // List<Note>
    private java.util.List notes_ = new java.util.ArrayList();
    private NameSectionStatus status_;

    /**
     * Creates a <code>NameSection</code>.
     *
     */
    public NameSection() {
        type_ = "";
    }

    /**
     * Creates a <code>NameSection</code>.
     *
     * @param source
     */
    public NameSection(NameSection source) {
        setup(source);
    }

    /**
     * Creates a <code>NameSection</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public NameSection(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>NameSection</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public NameSection(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>NameSection</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public NameSection(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>NameSection</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public NameSection(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>NameSection</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public NameSection(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>NameSection</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public NameSection(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>NameSection</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public NameSection(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>NameSection</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public NameSection(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>NameSection</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public NameSection(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>NameSection</code> by the NameSection <code>source</code>.
     *
     * @param source
     */
    public void setup(NameSection source) {
        int size;
        setType(source.getType());
        setId(source.getId());
        if (source.name_ != null) {
            setName((Name)source.getName().clone());
        }
        this.notes_.clear();
        size = source.notes_.size();
        for (int i = 0;i < size;i++) {
            addNote((Note)source.getNotes(i).clone());
        }
        if (source.status_ != null) {
            setStatus((NameSectionStatus)source.getStatus().clone());
        }
    }

    /**
     * Initializes the <code>NameSection</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>NameSection</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>NameSection</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        type_ = URelaxer.getAttributePropertyAsString(element, "type");
        id_ = URelaxer.getAttributePropertyAsString(element, "id");
        setName(new Name(stack));
       
        setStatus(new NameSectionStatus(stack));
        notes_.clear();
        while (true) {
            if (Note.isMatch(stack)) {
                addNote(new Note(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new NameSection(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("struct");
        int size;
        if (this.type_ != null) {
            URelaxer.setAttributePropertyByString(element, "type", this.type_);
        }
        if (this.id_ != null) {
            URelaxer.setAttributePropertyByString(element, "id", this.id_);
        }
        this.name_.makeElement(element);
        this.status_.makeElement(element);
        size = this.notes_.size();
        for (int i = 0;i < size;i++) {
            Note value = (Note)this.notes_.get(i);
            value.makeElement(element);
        }

        parent.appendChild(element);
    }

    /**
     * Initializes the <code>NameSection</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>NameSection</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>NameSection</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>NameSection</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>NameSection</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>NameSection</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>type</b>.
     *
     * @return String
     */
    public String getType() {
        return (type_);
    }

    /**
     * Sets the String property <b>type</b>.
     *
     * @param type
     */
    public void setType(String type) {
        this.type_ = type;
    }

    /**
     * Gets the String property <b>id</b>.
     *
     * @return String
     */
    public String getId() {
        return (id_);
    }

    /**
     * Sets the String property <b>id</b>.
     *
     * @param id
     */
    public void setId(String id) {
        this.id_ = id;
    }

    /**
     * Gets the NameSection property <b>name</b>.
     *
     * @return NameSection
     */
    public Name getName() {
        return (name_);
    }

    /**
     * Sets the NameSection property <b>name</b>.
     *
     * @param name
     */
    public void setName(Name name) {
        this.name_ = name;
    }

    /**
     * Gets the Note property <b>notes</b>.
     *
     * @return Note[]
     */
    public Note[] getNotes() {
        Note[] array = new Note[notes_.size()];
        return ((Note[])notes_.toArray(array));
    }

    /**
     * Sets the Note property <b>notes</b>.
     *
     * @param notes
     */
    public void setNotes(Note[] notes) {
        this.notes_.clear();
        for (int i = 0;i < notes.length;i++) {
            addNote(notes[i]);
        }
    }

    /**
     * Sets the Note property <b>notes</b>.
     *
     * @param notes
     */
    public void setNotes(Note notes) {
        this.notes_.clear();
        addNote(notes);
    }

    /**
     * Adds the Note property <b>notes</b>.
     *
     * @param notes
     */
    public void addNote(Note notes) {
        this.notes_.add(notes);
    }

    /**
     * Adds the Note property <b>notes</b>.
     *
     * @param notes
     */
    public void addNotes(Note[] notes) {
        for (int i = 0;i < notes.length;i++) {
            addNote(notes[i]);
        }
    }

    /**
     * Gets number of the Note property <b>notes</b>.
     *
     * @return int
     */
    public int sizeNotes() {
        return (notes_.size());
    }

    /**
     * Gets the Note property <b>notes</b> by index.
     *
     * @param index
     * @return Note
     */
    public Note getNotes(int index) {
        return ((Note)notes_.get(index));
    }

    /**
     * Sets the Note property <b>notes</b> by index.
     *
     * @param index
     * @param notes
     */
    public void setNote(int index, Note notes) {
        this.notes_.set(index, notes);
    }

    /**
     * Adds the Note property <b>notes</b> by index.
     *
     * @param index
     * @param notes
     */
    public void addNote(int index, Note notes) {
        this.notes_.add(index, notes);
    }

    /**
     * Remove the Note property <b>notes</b> by index.
     *
     * @param index
     */
    public void removeNote(int index) {
        this.notes_.remove(index);
    }

    /**
     * Remove the Note property <b>notes</b> by object.
     *
     * @param notes
     */
    public void removeNote(Note notes) {
        this.notes_.remove(notes);
    }

    /**
     * Clear the Note property <b>notes</b>.
     *
     */
    public void clearNotes() {
        this.notes_.clear();
    }

    /**
     * Gets the NameSectionStatus property <b>status</b>.
     *
     * @return NameSectionStatus
     */
    public NameSectionStatus getStatus() {
        return (status_);
    }

    /**
     * Sets the NameSectionStatus property <b>status</b>.
     *
     * @param status
     */
    public void setStatus(NameSectionStatus status) {
        this.status_ = status;
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<struct");
        if (type_ != null) {
            buffer.append(" type=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.append("\"");
        }
        if (id_ != null) {
            buffer.append(" id=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.append("\"");
        }
        buffer.append(">");
        name_.makeTextElement(buffer);
        status_.makeTextElement(buffer);
        size = this.notes_.size();
        for (int i = 0;i < size;i++) {
            Note value = (Note)this.notes_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</struct>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<struct");
        if (type_ != null) {
            buffer.write(" type=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.write("\"");
        }
        if (id_ != null) {
            buffer.write(" id=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.write("\"");
        }
        buffer.write(">");
        name_.makeTextElement(buffer);
        status_.makeTextElement(buffer);
        size = this.notes_.size();
        for (int i = 0;i < size;i++) {
            Note value = (Note)this.notes_.get(i);
            value.makeTextElement(buffer);
        }

        buffer.write("</struct>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<struct");
        if (type_ != null) {
            buffer.print(" type=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.print("\"");
        }
        if (id_ != null) {
            buffer.print(" id=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.print("\"");
        }
        buffer.print(">");
        name_.makeTextElement(buffer);
        status_.makeTextElement(buffer);
        size = this.notes_.size();
        for (int i = 0;i < size;i++) {
            Note value = (Note)this.notes_.get(i);
            value.makeTextElement(buffer);
        }

        buffer.print("</struct>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTypeAsString() {
        return (URelaxer.getString(getType()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getIdAsString() {
        return (URelaxer.getString(getId()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTypeByString(String string) {
        setType(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setIdByString(String string) {
        setId(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>NameSection</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "struct")) {
            return (false);
        }
        RStack target = new RStack(element);
        if (!URelaxer.hasAttributeHungry(target, "type")) {
            return (false);
        }
        if (!URelaxer.isMatchDataValuesAttr(element, "type", "token", "NS")) {
            return (false);
        }
        if (!Name.isMatchHungry(target)) {
            logger.info(NameSection.class + "-No name");
            return (false);
        }
        if (!NameSectionStatus.isMatchHungry(target)) {
            logger.info(NameSection.class+"-No NameSectionStatus");
            return (false);
        }
        while (true) {
            if (!Note.isMatchHungry(target)) {
                break;
            }
        }
        if (!target.isEmptyElement()) {
            logger.info(NameSection.class + "-NameSection NOT empty");
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>NameSection</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>NameSection</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
