package mpi.eudico.client.annotator.export;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Selection;
import mpi.eudico.server.corpora.clom.Transcription;

/**
 * A dialog that adds up/down buttons and all/none buttons to the dialog.
 * 
 * @author Han Sloetjes
 */
public abstract class AbstractExtTierExportDialog extends
		AbstractTierExportDialog {
	protected JPanel tierButtonPanel;
	protected JButton downButton;
	protected JButton upButton;
	protected JButton allButton;
	protected JButton noneButton;

	/**
	 * Constructor.
	 * @param parent parent frame
	 * @param modal modal flag
	 * @param transcription the transcription
	 * @param selection the selection
	 */
	public AbstractExtTierExportDialog(Frame parent, boolean modal,
			Transcription transcription, Selection selection) {
		super(parent, modal, transcription, selection);
	}

	@Override
	protected void makeLayout() {
		super.makeLayout();
		
        // add more 
        upButton = new JButton();
        downButton = new JButton();
        allButton = new JButton();
        noneButton = new JButton();
        tierButtonPanel = new JPanel(new GridBagLayout());
        
        try {
            ImageIcon upIcon = new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/navigation/Up16.gif"));
            ImageIcon downIcon = new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/navigation/Down16.gif"));
            upButton.setIcon(upIcon);
            downButton.setIcon(downIcon);
        } catch (Exception ex) {
            upButton.setText("Up");
            downButton.setText("Down");
        }
        upButton.addActionListener(this);
        downButton.addActionListener(this);
        allButton.addActionListener(this);
        noneButton.addActionListener(this);
        
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        tierButtonPanel.add(upButton, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        tierButtonPanel.add(downButton, gridBagConstraints);
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        gridBagConstraints.insets = new Insets(4, 20, 4, 2);
        tierButtonPanel.add(allButton, gridBagConstraints);
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        gridBagConstraints.insets = insets;
        tierButtonPanel.add(noneButton, gridBagConstraints);
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        tierSelectionPanel.add(tierButtonPanel, gridBagConstraints);
	}
	
    /**
     * The action performed event handling.
     *
     * @param ae the action event
     */
    public void actionPerformed(ActionEvent ae) {
        Object source = ae.getSource();

        if (source == upButton) {
            moveUp();
        } else if (source == downButton) {
            moveDown();
        } else if (source == allButton) {
            selectAll();
        } else if (source == noneButton) {
            selectNone();
        } else {
            super.actionPerformed(ae);
        }
    }

	@Override
	protected void updateLocale() {
		super.updateLocale();
		allButton.setText(ElanLocale.getString("Button.SelectAll"));
		noneButton.setText(ElanLocale.getString("Button.SelectNone"));
	}

    /**
     * Moves selected tiers up in the list of tiers.
     */
    protected void moveDown() {
        if ((tierTable == null) || (model == null) ||
                (model.getRowCount() < 2)) {
            return;
        }

        int[] selected = tierTable.getSelectedRows();

        for (int i = selected.length - 1; i >= 0; i--) {
            int row = selected[i];

            if ((row < (model.getRowCount() - 1)) &&
                    !tierTable.isRowSelected(row + 1)) {
                model.moveRow(row, row, row + 1);
                tierTable.changeSelection(row, 0, true, false);
                tierTable.changeSelection(row + 1, 0, true, false);
            }
        }
    }

    /**
     * Moves selected tiers up in the list of tiers.
     */
    protected void moveUp() {
        if ((tierTable == null) || (model == null) ||
                (model.getRowCount() < 2)) {
            return;
        }

        int[] selected = tierTable.getSelectedRows();

        for (int i = 0; i < selected.length; i++) {
            int row = selected[i];

            if ((row > 0) && !tierTable.isRowSelected(row - 1)) {
                model.moveRow(row, row, row - 1);
                tierTable.changeSelection(row, 0, true, false);
                tierTable.changeSelection(row - 1, 0, true, false);
            }
        }
    }
    
    /**
     * Set all tiers selected.
     */
    protected void selectAll() {
        if ((tierTable == null) || (model == null) ||
                (model.getRowCount() < 1)) {
            return;
        }
        int includeCol = model.findColumn(EXPORT_COLUMN);
        
        for (int i = 0; i < model.getRowCount(); i++) {
        	model.setValueAt(Boolean.TRUE, i, includeCol);
        }
    }
    
    /**
     * Set all tiers deselected.
     */
    protected void selectNone() {
        if ((tierTable == null) || (model == null) ||
                (model.getRowCount() < 1)) {
            return;
        }
        int includeCol = model.findColumn(EXPORT_COLUMN);
        
        for (int i = 0; i < model.getRowCount(); i++) {
        	model.setValueAt(Boolean.FALSE, i, includeCol);
        }
        tierTable.revalidate();
    }
}
