package mpi.eudico.client.annotator.export;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Selection;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.util.FileExtension;
import mpi.eudico.client.util.CheckBoxTableCellRenderer;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.praat.PraatTGEncoderInfo;
import mpi.eudico.server.corpora.clomimpl.praat.PraatTextGridEncoder;
import mpi.util.gui.FileAndEncodingChooser;


public class ExportPraatDialog extends AbstractExtTierExportDialog 
    implements ActionListener, ItemListener, ListSelectionListener{
    private JCheckBox rootTiersCB;
    private JCheckBox selectionCB;
    
    /**
     * @param parent the parent frame
     * @param modal the modal flag
     * @param transcription the transcription
     */
    public ExportPraatDialog(Frame parent, boolean modal,
            Transcription transcription, Selection selection) {
        super(parent, modal, transcription, selection);
        if (selection != null) {
            System.out.println("sb: " + selection.getBeginTime() + " se: " + selection.getEndTime());
        }
        makeLayout();
        extractTiers();
        postInit();
    }

    /**
     * The action performed event handling.
     *
     * @param ae the action event
     */
    public void actionPerformed(ActionEvent ae) {
        super.actionPerformed(ae);
    }

    /**
     * The item state changed handling.
     *
     * @param ie the ItemEvent
     */
    public void itemStateChanged(ItemEvent ie) {
        extractTiers();
    }

    /**
     * Updates the checked state of the export checkboxes.
     *
     * @param lse the list selection event
     */
    public void valueChanged(ListSelectionEvent lse) {
        if ((model != null) && lse.getValueIsAdjusting()) {
            int b = lse.getFirstIndex();
            int e = lse.getLastIndex();
            int col = model.findColumn(EXPORT_COLUMN);

            for (int i = b; i <= e; i++) {
                if (tierTable.isRowSelected(i)) {
                    model.setValueAt(Boolean.TRUE, i, col);
                }
            }
        }
    }

    /**
     * Extract candidate tiers for export.
     */
    protected void extractTiers() {
        if (model != null) {
            for (int i = model.getRowCount() - 1; i >= 0; i--) {
                model.removeRow(i);
            }

            if (transcription != null) {
                Vector v = transcription.getTiers();
                TierImpl t;
                boolean rootsOnly = rootTiersCB.isSelected();

                for (int i = 0; i < v.size(); i++) {
                    t = (TierImpl) v.get(i);

                    if (rootsOnly) {
                        // add only root tiers
                        if (t.getParentTier() == null) {
                            model.addRow(new Object[] { Boolean.TRUE, t.getName() });
                        }
                    } else {
                        // add all
                        if (t.getParentTier() == null) {
                            model.addRow(new Object[] { Boolean.TRUE, t.getName() });
                        } else {
                            model.addRow(new Object[] { Boolean.FALSE, t.getName() });
                        }
                    }
                }
            }

            if (model.getRowCount() > 1) {
                upButton.setEnabled(true);
                downButton.setEnabled(true);
            } else {
                upButton.setEnabled(false);
                downButton.setEnabled(false);
            }
        } else {
            upButton.setEnabled(false);
            downButton.setEnabled(false);
        }
    }

    
    /**
     * Initializes UI elements.
     */
    protected void makeLayout() {
        super.makeLayout();
        rootTiersCB = new JCheckBox();
        selectionCB = new JCheckBox();
        
        GridBagConstraints gridBagConstraints;

        rootTiersCB.setSelected(true);
        rootTiersCB.addItemListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        tierSelectionPanel.add(rootTiersCB, gridBagConstraints);

        model.setColumnIdentifiers(new String[] { EXPORT_COLUMN, TIER_NAME_COLUMN });

        tierTable.getColumn(EXPORT_COLUMN).setCellEditor(new DefaultCellEditor(
                new JCheckBox()));
        tierTable.getColumn(EXPORT_COLUMN).setCellRenderer(new CheckBoxTableCellRenderer());
        tierTable.getColumn(EXPORT_COLUMN).setMaxWidth(30);
        tierTable.setShowVerticalLines(false);
        tierTable.setTableHeader(null);
        tierTable.getSelectionModel().addListSelectionListener(this);
    
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = insets;
        optionsPanel.add(selectionCB, gridBagConstraints);

        updateLocale();
    }
    
    /**
     * Starts the actual export after performing some checks.
     *
     * @return true if export succeeded, false oherwise
     * @see mpi.eudico.client.annotator.export.AbstractBasicExportDialog#startExport()
     */
    protected boolean startExport() throws IOException {
        List selectedTiers = getSelectedTiers();

        if (selectedTiers.size() == 0) {
            JOptionPane.showMessageDialog(this,
                ElanLocale.getString("ExportTradTranscript.Message.NoTiers"),
                ElanLocale.getString("Message.Warning"),
                JOptionPane.WARNING_MESSAGE);

            return false;
        }

        String[] encodings = new String[]{ElanLocale.getString("Button.Default"), 
        		FileAndEncodingChooser.UTF_8, FileAndEncodingChooser.UTF_16};
        
        // prompt for file name and location
        File exportFile = promptForFile(ElanLocale.getString(
                    "ExportPraatDialog.Title"), FileExtension.PRAAT_TEXTGRID_EXT,
                    ElanFileFilter.createFileFilter(ElanFileFilter.PRAAT_TEXTGRID_TYPE), 
                    true, encodings);

        if (exportFile == null) {
            return false;
        }
        long begin = 0l;
        long end = ELANCommandFactory.getViewerManager(transcription).getMasterMediaPlayer().getMediaDuration();
        if (selectionCB.isSelected()) {
            if (selection != null && selection.getBeginTime() < selection.getEndTime()) {
                begin = selection.getBeginTime();
                end = selection.getEndTime();
            }
        }
        
        PraatTGEncoderInfo encInfo = new PraatTGEncoderInfo(begin, end);
        encInfo.setEncoding(encoding);
        
        PraatTextGridEncoder encoder = new PraatTextGridEncoder();
        encoder.encodeAndSave(transcription, encInfo, selectedTiers, 
                exportFile.getAbsolutePath());
        
        return true;
    }
    
    /**
     * Applies localized strings to the ui elements. For historic reasons the
     * string identifiers start with "TokenizeDialog"
     */
    protected void updateLocale() {
        super.updateLocale();
        setTitle(ElanLocale.getString("ExportPraatDialog.Title"));
        titleLabel.setText(ElanLocale.getString("ExportPraatDialog.Title"));
        rootTiersCB.setText(ElanLocale.getString(
                "ExportTradTranscript.Label.RootTiers"));
        selectionCB.setText(ElanLocale.getString("ExportDialog.Restrict"));
    }

    
}
