package mpi.eudico.client.annotator.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import mpi.eudico.client.annotator.ElanFrame2;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.dobes.ACMTranscriptionStore;
import mpi.eudico.client.annotator.commands.global.OpenMA;
import mpi.eudico.client.annotator.gui.MultiFileChooser;
import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clom.TranscriptionStore;
import mpi.eudico.client.annotator.util.FileUtility;
import mpi.eudico.server.corpora.clomimpl.abstr.MediaDescriptor;
import mpi.eudico.server.corpora.clomimpl.abstr.LinkedFileDescriptor;

/**
 * This dialog is used for exporting multiple eaf-files to new eaf-files
 * in which there the tiers to be exported can be chosen as well as the resulting
 * file names.
 * @author Jeffrey Lemein
 * @version March 2010
 */
public class ExportTiersDialog extends ClosableDialog 
		implements ListSelectionListener, ClientLogger {
	private JFrame parent;
	private ArrayList<TranscriptionImpl> transImplList;
	private File[] eafFiles;
	private int progressValue;
	private boolean filesOpened, exportingStarted;
	
	//utilities
	private ButtonGroup exportButtonGroup, dependentTiersBtnGroup, saveTierBtnGroup, fileNameBtnGroup, suffixBtnGroup;
	
	//components
	private JLabel descriptionLabel, progressBarLabel, numberOfFilesLabel;
	private JTable tierTable, linguisticTypeTable;
	private JButton selectAllButton, selectNoneButton, exportButton, cancelButton, browseBtn;
	private JCheckBox dontExportFilesWithoutTiersCB;
	private JLabel dependentTiersLabel, directoryOptionsLabel, exportLabel, resultingFilenameLabel, otherOptionLabel;
	private JRadioButton exportTiersRB, exportLingTypesRB, includeParentTiersRB, neglectDependentTiersRB, originalDirRB, togetherInSameDirRB, newDirectoryRB;
	private JRadioButton originalFileNameRB, addSuffixRB, originalFileNameWithSuffixRB, useBaseNameWithSuffixRB;
	private JScrollPane tierListScrollPane, linguisticTypeScrollPane;
	private JTabbedPane tabbedPane;
	private JTextField sameDirectoryTextField, localDirectoryTextField, suffixNameTextField, baseNameTextField;
	private JProgressBar progressBar;
	private JPanel tierListPanel, tierListDetailsPanel, selectButtonPanel, optionPanel, exportButtonsPanel;
	private JPanel exportPanel, dependentTiersOptionPanel, fileNameOptionsPanel, directoryOptionsPanel, otherOptionsPanel;
	private String browseDirText;
	
	/**
	 * Initializes dialog, asks user for selecting files to export and loads
	 * these files into the dialog
	 * @param parentframe the parent frame of this dialog
	 */
	public ExportTiersDialog(JFrame parentFrame, List<File> files){
		super(parentFrame);
		parent = parentFrame;
		filesOpened = false;
		exportingStarted = false;

		if (files == null) {
			return;
		} else {
			boolean inited = initFilesArray(files);
			if (!inited) {
				setVisible(false);
				dispose();
				return;
			}
		}
		
		//if the user did not select files from the multiple file chooser, then stop exporting
//		if( !showMultiFileChooser() ){
//			setVisible(false);
//			dispose();
//			return;
//		}
		

		//configure data for GUI components and lay them out on the dialog
		initComponents();
		layoutComponents();
		
		//open/load eaf files selected in multiple file chooser
		loadEAFs();
		
		//set language specific titles/captions
		updateLocale();
		
		setVisible(true);
		
		//resize and relocate window
		pack();
		setLocationRelativeTo(parent);
	}
	
	/**
	 * initializes the components that are placed in the dialog (but doesn't align them)
	 */
	private void initComponents()
	{
		//create and edit font/alignment of topmost description label
		descriptionLabel = new JLabel();
		descriptionLabel.setFont(descriptionLabel.getFont().deriveFont(16.0f));
		
		/*
		 * Panel with tier list and select all/none buttons
		 */
		tierListPanel = new JPanel();
		tierTable = new JTable( new TierTableModel() );
		tierTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		tierTable.getSelectionModel().addListSelectionListener(this);
		tierTable.setShowVerticalLines(false);
		tierTable.setTableHeader(null);
		
		tierListScrollPane = new JScrollPane(tierTable);
		tierListScrollPane.setPreferredSize(new Dimension(180, 100));
		tierListScrollPane.setColumnHeaderView(null);
		
		//Button panel under list of selectable tiers
		tierListDetailsPanel = new JPanel();
		numberOfFilesLabel = new JLabel();
		selectButtonPanel = new JPanel();
		selectAllButton = new JButton();
		selectNoneButton = new JButton();
		
		linguisticTypeTable = new JTable( new TierTableModel() );
		linguisticTypeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		linguisticTypeTable.getSelectionModel().addListSelectionListener(this);
		linguisticTypeTable.setShowVerticalLines(false);
		linguisticTypeTable.setTableHeader(null);
		
		linguisticTypeScrollPane = new JScrollPane(linguisticTypeTable);
		linguisticTypeScrollPane.setPreferredSize(new Dimension(180, 100));
		linguisticTypeScrollPane.setColumnHeaderView(null);
		
		tabbedPane = new JTabbedPane();
		tabbedPane.addTab("", tierListScrollPane);
		tabbedPane.addTab("", linguisticTypeScrollPane);
		/* */	

		//optionPanel consists of all the panels with options (see below)
		optionPanel = new JPanel();	
		
		/*
		 * Panel with export options
		 */
		exportPanel = new JPanel();
		exportTiersRB = new JRadioButton("", true);
		exportLingTypesRB = new JRadioButton();
		exportButtonGroup = new ButtonGroup();
		exportButtonGroup.add(exportTiersRB);
		exportButtonGroup.add(exportLingTypesRB);
		/**/
		
		/*
		 * Panel with dependent tier options
		 */
		dependentTiersOptionPanel = new JPanel();
		dependentTiersLabel = new JLabel();
		includeParentTiersRB = new JRadioButton("", true);
		neglectDependentTiersRB = new JRadioButton();
		dependentTiersBtnGroup = new ButtonGroup();
		dependentTiersBtnGroup.add(includeParentTiersRB);
		dependentTiersBtnGroup.add(neglectDependentTiersRB);
		/* */
		
		/*
		 * Panel with file name options
		 */
		fileNameOptionsPanel = new JPanel();
		
		resultingFilenameLabel = new JLabel();
		
		originalFileNameRB = new JRadioButton("", true);
		addSuffixRB = new JRadioButton();
		fileNameBtnGroup = new ButtonGroup();
		fileNameBtnGroup.add(originalFileNameRB);
		fileNameBtnGroup.add(addSuffixRB);
		
		suffixBtnGroup = new ButtonGroup();
		originalFileNameWithSuffixRB = new JRadioButton("", true);
		originalFileNameWithSuffixRB.setEnabled(false);
		suffixNameTextField = new JTextField();
		suffixNameTextField.setEnabled(false);
		
		useBaseNameWithSuffixRB = new JRadioButton();
		useBaseNameWithSuffixRB.setEnabled(false);
		baseNameTextField = new JTextField();
		baseNameTextField.setEnabled(false);		
		
		suffixBtnGroup.add(originalFileNameWithSuffixRB);
		suffixBtnGroup.add(useBaseNameWithSuffixRB);
		/* */
		
		/*
		 * Panel with directory options
		 */
		directoryOptionsPanel = new JPanel();
		
		directoryOptionsLabel = new JLabel();
		
		originalDirRB = new JRadioButton("", true);
		newDirectoryRB = new JRadioButton();
		togetherInSameDirRB = new JRadioButton();
		
		saveTierBtnGroup = new ButtonGroup();
		saveTierBtnGroup.add(originalDirRB);
		saveTierBtnGroup.add(togetherInSameDirRB);
		saveTierBtnGroup.add(newDirectoryRB);
		
		localDirectoryTextField = new JTextField();
		localDirectoryTextField.setEnabled(false);
		
		sameDirectoryTextField = new JTextField();
		sameDirectoryTextField.setEnabled(false);
		sameDirectoryTextField.setEditable(false);
		
		browseBtn = new JButton();
		browseBtn.setEnabled(false);
		/* */
		
		/*
		 * Additional Options Label
		 */
		otherOptionsPanel = new JPanel();
		
		otherOptionLabel = new JLabel();
		
		dontExportFilesWithoutTiersCB = new JCheckBox();
		/* */
		
		//Progress bar
		progressBar = new JProgressBar(0, eafFiles.length);
		progressBar.setValue(progressValue);
		progressBar.setStringPainted(true);
		progressValue = 0;
		
		//Export buttons
		exportButtonsPanel = new JPanel();
		exportButton = new JButton();
		exportButton.setEnabled(false);
		cancelButton = new JButton();
		
		//event handlers
		ActionListener radioBtnListener = new RadioButtonHandler();
		ActionListener buttonListener = new ButtonHandler();
		KeyListener keyListener = new TextFieldHandler();
		
		//event handlers for the radio buttons
		originalDirRB.addActionListener( radioBtnListener );
		newDirectoryRB.addActionListener( radioBtnListener );
		togetherInSameDirRB.addActionListener( radioBtnListener );
		originalFileNameRB.addActionListener( radioBtnListener );
		addSuffixRB.addActionListener( radioBtnListener );
		originalFileNameWithSuffixRB.addActionListener( radioBtnListener );
		useBaseNameWithSuffixRB.addActionListener( radioBtnListener );
		
		//event handlers for the buttons
		browseBtn.addActionListener( buttonListener );
		exportButton.addActionListener( buttonListener );
		cancelButton.addActionListener( buttonListener );
		selectAllButton.addActionListener( buttonListener );
		selectNoneButton.addActionListener( buttonListener );
		
		//event handlers for the text fields
		suffixNameTextField.addKeyListener( keyListener );
		baseNameTextField.addKeyListener( keyListener );
		localDirectoryTextField.addKeyListener( keyListener );
	}
	
	/**
	 * Specifies a layout manager and adds all the components to the form
	 */
	private void layoutComponents(){
		//insets used to align components
		Insets noTabInset = new Insets(0, 5, 3, 4);
		Insets singleTabInset = new Insets(0, 15, 3, 0);
		Insets doubleTabInset = new Insets(0, 35, 3, 0);
		
		GridBagConstraints gbc;
		
		//label in top
		descriptionLabel.setHorizontalAlignment(SwingConstants.CENTER);
		descriptionLabel.setPreferredSize(new Dimension(descriptionLabel.getWidth(), 25));
		
		/*
		 * Tier list panel
		 */
		tierListPanel.setLayout( new BorderLayout() );
		
		tierListDetailsPanel.setLayout(new GridBagLayout());
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = noTabInset;
		gbc.weightx = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		tierListDetailsPanel.add(exportPanel, gbc);
		
		gbc.gridy = 1;
		gbc.anchor = GridBagConstraints.CENTER;
		tierListDetailsPanel.add(numberOfFilesLabel, gbc);
		
		gbc.gridy = 2;
		gbc.anchor = GridBagConstraints.CENTER;
		tierListDetailsPanel.add(selectButtonPanel, gbc);
		
		selectButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		selectButtonPanel.add(selectAllButton);
		selectButtonPanel.add(selectNoneButton);
	
		tierListPanel.add(tabbedPane, BorderLayout.CENTER);
		tierListPanel.add(tierListDetailsPanel, BorderLayout.SOUTH);
		/* */
		
		/*
		 * Export Options Panel
		 */
		exportPanel.setLayout(new GridBagLayout());
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = noTabInset;
		exportPanel.add( exportTiersRB, gbc );
		
		gbc.gridy = 1;
		gbc.insets = noTabInset;
		exportPanel.add( exportLingTypesRB, gbc );
		/**/
		
		/*
		 * Dependent Tiers options panel
		 */
		dependentTiersOptionPanel.setLayout(new GridBagLayout());
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = noTabInset;
		dependentTiersOptionPanel.add(dependentTiersLabel, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.insets = singleTabInset;
		dependentTiersOptionPanel.add(includeParentTiersRB, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.insets = singleTabInset;
		dependentTiersOptionPanel.add(neglectDependentTiersRB, gbc);
		/* */
		
		/*
		 * File name option panel
		 */
		fileNameOptionsPanel.setLayout( new GridBagLayout() );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = noTabInset;
		fileNameOptionsPanel.add(resultingFilenameLabel, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 0;
		gbc.insets = singleTabInset;
		fileNameOptionsPanel.add(originalFileNameRB, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.insets = singleTabInset;
		fileNameOptionsPanel.add(addSuffixRB, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.gridwidth = 1;
		gbc.weightx = 0;
		gbc.insets = doubleTabInset;
		fileNameOptionsPanel.add(originalFileNameWithSuffixRB, gbc);
		
		gbc.gridx = 1;
		gbc.gridy = 3;
		gbc.insets = noTabInset;
		gbc.weightx = 1.0;
		fileNameOptionsPanel.add(suffixNameTextField, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 4;
		gbc.gridwidth = 1;
		gbc.weightx = 0;
		gbc.insets = doubleTabInset;
		fileNameOptionsPanel.add(useBaseNameWithSuffixRB, gbc);
		
		gbc.gridx = 1;
		gbc.gridy = 4;
		gbc.insets = noTabInset;
		gbc.weightx = 1.0;
		fileNameOptionsPanel.add(baseNameTextField, gbc);
		/* */
		
		/*
		 * Directory Options Panel
		 */
		directoryOptionsPanel.setLayout( new GridBagLayout() );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridwidth = 3;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = noTabInset;
		directoryOptionsPanel.add(directoryOptionsLabel, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridwidth = 3;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = singleTabInset;
		directoryOptionsPanel.add(originalDirRB, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 0;
		gbc.insets = singleTabInset;
		directoryOptionsPanel.add(newDirectoryRB, gbc);
		
		gbc.gridx = 1;
		gbc.gridy = 2;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		gbc.weightx = 0;
		gbc.insets = noTabInset;
		directoryOptionsPanel.add(localDirectoryTextField, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 0;
		gbc.insets = singleTabInset;
		directoryOptionsPanel.add(togetherInSameDirRB, gbc);
		
		gbc.gridx = 1;
		gbc.gridy = 3;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = noTabInset;
		sameDirectoryTextField.setMinimumSize(new Dimension( 194, sameDirectoryTextField.getMinimumSize().height));
		directoryOptionsPanel.add(sameDirectoryTextField, gbc);
		
		gbc.gridx = 2;
		gbc.gridy = 3;
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 0;
		gbc.insets = noTabInset;
		directoryOptionsPanel.add(browseBtn, gbc);
		/* */
		
		/*
		 * Other options panel
		 */
		otherOptionsPanel.setLayout( new GridBagLayout() );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = noTabInset;
		otherOptionsPanel.add(otherOptionLabel, gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.insets = singleTabInset;
		otherOptionsPanel.add(dontExportFilesWithoutTiersCB, gbc);
		/* */
		
		/*
		 * Export Buttons Panel
		 */
		exportButtonsPanel.setLayout( new FlowLayout(FlowLayout.RIGHT) );
		exportButton.setSize(cancelButton.getSize());
		exportButtonsPanel.add(exportButton);
		exportButtonsPanel.add(cancelButton);
		/* */
		
		//Add all panels to the option panel		
		optionPanel.setLayout( new BoxLayout(optionPanel, BoxLayout.Y_AXIS) );
		optionPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		optionPanel.add(dependentTiersOptionPanel);
		optionPanel.add( Box.createRigidArea(new Dimension(0, 7)));
		optionPanel.add(fileNameOptionsPanel);
		optionPanel.add( Box.createRigidArea(new Dimension(0, 7)));
		optionPanel.add(directoryOptionsPanel);
		optionPanel.add( Box.createRigidArea(new Dimension(0, 7)));
		optionPanel.add(otherOptionsPanel);
		
		/*
		 * Container elements
		 */
		JPanel container = new JPanel( new GridBagLayout() );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		gbc.weighty = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.PAGE_START;
		container.add(descriptionLabel, gbc );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.PAGE_START;
		container.add(tierListPanel, gbc );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.fill = GridBagConstraints.BOTH;
		container.add( optionPanel, gbc );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		gbc.weighty = 0;
		gbc.insets = new Insets(10, 10, 10, 10);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.LAST_LINE_START;
		container.add( progressBar, gbc );
		
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.PAGE_END;
		container.add(exportButtonsPanel, gbc );
		/* */
		
		//add the container to the dialog
		getContentPane().setLayout( new GridBagLayout() );
		Insets insets = new Insets(4, 8, 3, 7);
		gbc = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, insets, 0, 0);
		getContentPane().add(container, gbc);
	}
	
	/**
	 * Function that updates the table with the choosable tier names and linguistic types. 
	 * Method will be called after all eaf-files are opened and tiernames/linguistic types are extracted.
	 * @param tierNames the list of choosable tier names (that needs to be showed in the table)
	 * @param linguisticTypes the list of linguistic types
	 */
	private void updateTierList( ArrayList<String> tierNames, ArrayList<String> linguisticTypes ){
		//update tier table with new model
		tierTable.setModel( new TierTableModel( tierNames ) );
		
		if( tierTable.getColumnCount() == 2 )
			tierTable.getColumnModel().getColumn(0).setMaxWidth(30);
		
		//update linguistic type table with new model
		linguisticTypeTable.setModel( new TierTableModel( linguisticTypes ) );
		
		if( linguisticTypeTable.getColumnCount() == 2 )
			linguisticTypeTable.getColumnModel().getColumn(0).setMaxWidth(30);
		
		filesOpened = true;
		updateExportButtonEnabled();
	}
	
	/**
	 * Method that updates the export button according to the state of selected options and opening of files
	 */
	private void updateExportButtonEnabled(){
		if( !filesOpened ){
			exportButton.setEnabled(false);
			return;
		}
		
		boolean b = true;
		
		if( togetherInSameDirRB.isSelected() && sameDirectoryTextField.getText().equals(browseDirText) )
			b = false;
		
		if( newDirectoryRB.isSelected() && localDirectoryTextField.getText().length() <= 0 )
			b = false;
		
		if( addSuffixRB.isSelected() ){
			if( originalFileNameWithSuffixRB.isSelected() && suffixNameTextField.getText().length() <= 0 )
				b = false;
			else if( useBaseNameWithSuffixRB.isSelected() && baseNameTextField.getText().length() <= 0 )
				b = false;
		}
		
		exportButton.setEnabled(b && !exportingStarted);
	}
	
	/**
	 * Removes all duplicates from the list
	 * @param list the list containing duplicates that need to be removed
	 */
	private void removeDuplicates(List list){
		 Set set = new HashSet();
		 List newList = new ArrayList();
		 
		 for (Iterator iter = list.iterator(); iter.hasNext(); ){
			 Object element = iter.next();
			 if (set.add(element))
				 newList.add(element);
		 }
		 
		 list.clear();
		 list.addAll(newList);
	}

	/**
	 * Updates the text and captions of the components
	 */
	private void updateLocale(){
		setTitle( ElanLocale.getString("ExportTiersDialog.Title") );
		this.descriptionLabel.setText(ElanLocale.getString("ExportTiersDialog.Label.Description"));
		this.tierListPanel.setBorder( new TitledBorder( ElanLocale.getString("ExportTiersDialog.List.Title" ) ) );
		this.optionPanel.setBorder( new TitledBorder( ElanLocale.getString("ExportTiersDialog.OptionPanel.Title") ));
		this.exportPanel.setBorder( new TitledBorder( ElanLocale.getString("ExportTiersDialog.Title.ExportOptions") ));
		this.neglectDependentTiersRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.NeglectDependentTiers") );
		this.includeParentTiersRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.IncludeParentTiersRB") );
		this.exportButton.setText( ElanLocale.getString("ExportTiersDialog.Button.Export"));
		this.cancelButton.setText( ElanLocale.getString("Button.Cancel"));
		this.selectAllButton.setText( ElanLocale.getString( "Button.SelectAll" ) );
		this.selectNoneButton.setText( ElanLocale.getString( "Button.SelectNone" ) );
		
		this.originalFileNameRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.OriginalFileName"));
		this.originalFileNameWithSuffixRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.OriginalFileNameWithSuffix"));
		this.useBaseNameWithSuffixRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.NewBaseNameWithSuffix"));
		this.numberOfFilesLabel.setText(ElanLocale.getString("ExportTiersDialog.Label.NumberOfFiles") + " " + eafFiles.length);
		
		this.originalDirRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.OriginalDirectory"));
		this.newDirectoryRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.NewDirectory"));
		this.togetherInSameDirRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.TogetherInSameDirectory"));
		
		this.browseBtn.setText( ElanLocale.getString("ExportTiersDialog.Button.Browse") );
		
		this.baseNameTextField.setText( ElanLocale.getString("ExportTiersDialog.TextField.DefaultBaseName") );
		this.suffixNameTextField.setText( ElanLocale.getString("ExportTiersDialog.TextField.DefaultSuffixName") );
		this.localDirectoryTextField.setText( ElanLocale.getString("ExportTiersDialog.TextField.DefaultLocalDirectoryName") );
		browseDirText = ElanLocale.getString("ExportTiersDialog.TextField.DirectoryNameField");
		this.sameDirectoryTextField.setText( browseDirText );
		
		this.dependentTiersLabel.setText( ElanLocale.getString("ExportTiersDialog.Label.DependentTierOptions"));
		this.directoryOptionsLabel.setText( ElanLocale.getString("ExportTiersDialog.Label.SaveTiersOptions") );
		this.resultingFilenameLabel.setText( ElanLocale.getString("ExportTiersDialog.Label.FileNameOptions") );
		this.otherOptionLabel.setText( ElanLocale.getString("ExportTiersDialog.Label.OtherOptions") );
		
		this.tabbedPane.setTitleAt(0, ElanLocale.getString("ExportTiersDialog.Tab1"));
		this.tabbedPane.setTitleAt(1, ElanLocale.getString("ExportTiersDialog.Tab2"));
		this.exportTiersRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.ExportTiers") );
		this.exportLingTypesRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.ExportLingTypes") );
		
		this.dontExportFilesWithoutTiersCB.setText( ElanLocale.getString("ExportTiersDialog.CheckBox.ExportFilesWithoutTiers"));
		this.addSuffixRB.setText( ElanLocale.getString("ExportTiersDialog.RadioButton.addSuffix"));
	}
	
	private boolean initFilesArray(List<File> files) {
        //assume that all selected files exist
        boolean allFilesExist = true;

        int numExist = 0;
        for (File f : files) {
        	if (f == null) {
        		continue;
        	}
        	if (f.exists()) {
        		numExist++;
        	} else {
        		allFilesExist = false;
        	}
        }
        
        if (!allFilesExist) {
        	//ask if user wants to continue with exporting, if some of the files doesn't exist.
        	int answer = JOptionPane.showConfirmDialog(this, 
        			ElanLocale.getString("ExportTiersDialog.Message.NotAllFilesExist"), 
        			ElanLocale.getString("ExportTiersDialog.Message.NotAllFilesExistTitle"), 
        			JOptionPane.YES_NO_OPTION );
        	
        	//stop with exporting if user doesn't want to continue
        	if( answer != JOptionPane.YES_OPTION ) {
        		return false;
        	}       		
        }
        
        eafFiles = new File[numExist];
        int index = 0;
        for (File f : files) {
        	if (f == null) {
        		continue;
        	}
        	if (f.exists()) {
        		eafFiles[index] = f;
        		index++;
        	}
        }
        
        return true;
	}
	
	/**
	 * Shows a multiple file chooser dialog, checks if every selected file exists
	 * and stores the selected files in private variable eafFiles
	 * @return boolean to indicate if file selection went successful
	 */
	private boolean showMultiFileChooser(){
		//retrieve last used directory or use default one
        String dirPath = (String) Preferences.get("LastUsedEAFDir", null);

        if (dirPath == null)
            dirPath = System.getProperty("user.dir");

        //create a multifile chooser and set relevant attributes
        MultiFileChooser multiFileChooser = new MultiFileChooser();   
        multiFileChooser.setCurrentDirectory( new File(dirPath));
        multiFileChooser.setDialogTitle( ElanLocale.getString("Frame.ElanFrame.OpenDialog.Title"));
        
        //only accept eaf file types
        multiFileChooser.setFileFilter( ElanFileFilter.createFileFilter( ElanFileFilter.EAF_TYPE ));
		
		//show multiple file chooser and store result
        int returnValue = multiFileChooser.showDialog(this, "OK");

		if (returnValue == JFileChooser.APPROVE_OPTION) {
			//get the selected files that the user has chosen in the multiple file dialog
            Object[] objects = multiFileChooser.getFiles();
            
            //convert all objects to the file array and checks if all files exist
            //this extra step is done because mutliFileChooser.getFiles()
            //can't be casted to a File array directly (ClassCastException)
            eafFiles = new File[objects.length];
            
            //assume that all selected files exist
            boolean allFilesExist = true;
            
            for( int i=0; i<objects.length; i++ ){
            	File file = (File) objects[i];
            	
            	//check if file exists
            	if( file.exists() )
            		eafFiles[i] = file;
            	else
            		//flag that one of the files does not exist
            		allFilesExist = false;
            }
            
            
            if( !allFilesExist ){
            	//ask if user wants to continue with exporting, if some of the files doesn't exist.
            	int answer = JOptionPane.showConfirmDialog(this, ElanLocale.getString("ExportTiersDialog.Message.NotAllFilesExist"), ElanLocale.getString("ExportTiersDialog.Message.NotAllFilesExistTitle"), JOptionPane.YES_NO_OPTION );
            	
            	//stop with exporting if user doesn't want to continue
            	if( answer != JOptionPane.YES_OPTION )
            		return false;
            }

            //store last used directory
            Preferences.set("LastUsedEAFDir",
                (new File(eafFiles[eafFiles.length-1].getAbsolutePath())).getParent(),
                null);

            return true;
        }else{
        	//if user aborts, then remove dialog
        	setVisible(false);
        	dispose();
        	
        	return false;
        }
	}
	
	/**
	 * shows a dialog that lets the user choose a directory
	 * @return string representing the selected directory name or null if no directory has been chosen
	 */
	private String showDirectoryChooser(){
        String dirPath = (String) Preferences.get("LastUsedEAFDir", null);

        //use default directory if last used directory does not exist
        if (dirPath == null)
            dirPath = System.getProperty("user.dir");
        
        //create a directory chooser and set relevant attributes
        JFileChooser dirChooser = new JFileChooser();
        
        dirChooser.setCurrentDirectory( new File(dirPath) );
        dirChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        dirChooser.setDialogTitle( ElanLocale.getString("Frame.ElanFrame.OpenDialog.Title") );
		
        //show the directory chooser and store the result
		int returnVal = dirChooser.showDialog(this, "OK");
		
		//if OK button has been pressed in directory chooser
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			//get selected directory
            File selectedDir = dirChooser.getSelectedFile();
            
            //if selected directory does not exist, let the user choose again
            if( !selectedDir.exists() ){
            	JOptionPane.showMessageDialog(this, ElanLocale.getString("ExportTiersDialog.Message.DirectoryDoesntExist"), ElanLocale.getString("ExportTiersDialog.Message.DirectoryDoesntExistTitle"), JOptionPane.OK_OPTION );
            	
            	return showDirectoryChooser();
            }

            //set last used directory
            Preferences.set("LastUsedEAFDir",
                (new File(eafFiles[eafFiles.length-1].getAbsolutePath())).getParent(),
                null);
            
            //return the selected directory
            return selectedDir.getAbsolutePath();
        }else{
        	//user aborted selecting a directory, so return null
        	return null;
        }
	}
	
	/**
	 * open files (stored in eafFiles[]) and extract all tiernames(in a separate thread) 
	 */
	private void loadEAFs(){
		Thread t = new OpenFilesThread();
		t.start();
	}
	
	/**
	 * Method that saves the opened eafFiles with the given list of tiers. All tiers
	 * that are not in tiersToSave won't be saved.
	 * @param tiersToSave names of tiers that need to be saved 
	 */
	private void saveFiles( List<String> tiersToSave ){
		Thread t = new SaveFileThread(tiersToSave);
		t.start();
	}

	/**
	 * Helper function to check if a given string exists in the list of strings
	 * @param string particular string that needs to be tested if it occurs in the list
	 * @param stringList the list of strings
	 * @return
	 */
	private boolean exists( String string, ArrayList<String> stringList ){
		return stringList.indexOf(string) >= 0;
	}

	/**
	 * Gives the active (or visible) table that is shown in the tabbedPane panel
	 * @return JTable that is currently visible in the tabbedPane panel
	 */
	private JTable getActiveTable(){
		final int TIER_TAB = 0, LINGUISTIC_TYPE_TAB = 1;
		
		switch( tabbedPane.getSelectedIndex() ){
			case TIER_TAB:
				return tierTable;
				
			case LINGUISTIC_TYPE_TAB:
				return linguisticTypeTable;
				
			default:
				return null;
		}
	}
	/**
	 * Handler for the case if a user selects a tier in the table.
	 * It then sets the value in the table accordingly
	 */
	public void valueChanged(ListSelectionEvent e) {		
		ListSelectionModel lsm = (ListSelectionModel) e.getSource();

		if( e.getValueIsAdjusting() ){
			int selectedRow = lsm.getMinSelectionIndex();
			
			JTable activeTable = getActiveTable();
			
			//only continue if selectedRow is greater than or equal to 0
			if( selectedRow >= 0 )
				activeTable.setValueAt( !(Boolean)activeTable.getValueAt(selectedRow, 0), selectedRow, 0);
		}
	}
	
	/**
	 * ActionListener that handles action events of all radio buttons in this dialog
	 * @author Jeffrey Lemein
	 * @version March 2010
	 */
	private class RadioButtonHandler implements ActionListener{
		/**
		 * handles all events of radio button clicks
		 */
		public void actionPerformed(ActionEvent e) {
			sameDirectoryTextField.setEnabled( togetherInSameDirRB.isSelected() );
			browseBtn.setEnabled( togetherInSameDirRB.isSelected() );
			
			localDirectoryTextField.setEnabled(newDirectoryRB.isSelected());
			baseNameTextField.setEnabled( addSuffixRB.isSelected() && useBaseNameWithSuffixRB.isSelected());
			suffixNameTextField.setEnabled( addSuffixRB.isSelected() && originalFileNameWithSuffixRB.isSelected() );
			useBaseNameWithSuffixRB.setEnabled( addSuffixRB.isSelected() );
			originalFileNameWithSuffixRB.setEnabled( addSuffixRB.isSelected() );
			
			updateExportButtonEnabled();
		}
	}
	
	/**
	 * KeyListener that handles key released events of the textfield
	 * if this textfield doesn't contain any character, then disable exportButton
	 * @author Jeffrey Lemein
	 * @version March 2010
	 */
	private class TextFieldHandler implements KeyListener{

		public void keyPressed(KeyEvent e) {}

		public void keyReleased(KeyEvent e) {
			updateExportButtonEnabled();
		}

		public void keyTyped(KeyEvent e) {}
		
	}
	
	/**
	 * ActionListener that handles all action events from the buttons in this dialog
	 * @author Jeffrey Lemein
	 * @version March 2010
	 */
	private class ButtonHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JButton sender = (JButton) e.getSource();
			
			if( sender == cancelButton ){				
				ExportTiersDialog.this.setVisible(false);
	            ExportTiersDialog.this.dispose();
	            return;
			}
			
			if( sender == browseBtn ){
				String directoryStr = showDirectoryChooser();
				
				if( directoryStr != null ){
					sameDirectoryTextField.setText( directoryStr );
					updateExportButtonEnabled();
				}
				
				return;
			}
			
			if( sender == exportButton ){
				//reset progress bar
				progressValue = 0;
				progressBar.setValue(progressValue);
				
				//begin exporting
				if( exportTiersRB.isSelected() )
					saveFiles( ((TierTableModel)tierTable.getModel()).getSelectedTiers() );
				else if( exportLingTypesRB.isSelected() )
					saveFiles( ((TierTableModel)linguisticTypeTable.getModel()).getSelectedTiers() );
				
				exportingStarted = true;	
	            updateExportButtonEnabled();
	            
				return;
			}
			
			if( sender == selectAllButton || sender == selectNoneButton ){
				Boolean newValue = sender == selectAllButton ? true : false;
				
				JTable activeTable = getActiveTable();
				
				if( activeTable.getColumnCount() > 0 && activeTable.getColumnClass(0) == newValue.getClass() )
					//select or deselect all tiers depending on newValue
					for( int row=0; row<activeTable.getRowCount(); row++ )
						activeTable.setValueAt(newValue, row, 0);
				
				//update the tier table
				activeTable.repaint();
				return;
			}
		}
	}

	/**
	 * Thread used to save files from eafFiles in a separate thread
	 * @author Jeffrey Lemein
	 * @version March 2010
	 */
	private class SaveFileThread extends Thread{
		private ArrayList<String> tiersToSave;
		private final int YES = 1, NO = 2, YES_TO_ALL = 0, NO_TO_ALL = 3;
		private final boolean originalDirRB, newDirectoryRB, togetherInSameDirRB, originalFileNameRB, originalFileNameWithSuffixRB, useBaseNameWithSuffixRB, neglectDependentTiersRB, dontExportFilesWithoutTiersCB, exportTiersRB, exportLingTypesRB;
		private final String baseNameTextField, suffixNameTextField, localDirectoryTextField, sameDirectoryTextField;
		
		/**
		 * Saves all files with the tier names in the given list
		 * @param tiersToSave the names of the tiers that need to be saved
		 */
		public SaveFileThread( List<String> tiersToSave ){
			this.tiersToSave = (ArrayList<String>) tiersToSave;
			
			//store the isSelected state, so that the states can't change during export
			originalDirRB = ExportTiersDialog.this.originalDirRB.isSelected();
			newDirectoryRB = ExportTiersDialog.this.newDirectoryRB.isSelected();
			togetherInSameDirRB = ExportTiersDialog.this.togetherInSameDirRB.isSelected();
			originalFileNameRB = ExportTiersDialog.this.originalFileNameRB.isSelected();
			originalFileNameWithSuffixRB = ExportTiersDialog.this.originalFileNameWithSuffixRB.isSelected();
			useBaseNameWithSuffixRB = ExportTiersDialog.this.useBaseNameWithSuffixRB.isSelected();
			neglectDependentTiersRB = ExportTiersDialog.this.neglectDependentTiersRB.isSelected();
			dontExportFilesWithoutTiersCB = ExportTiersDialog.this.dontExportFilesWithoutTiersCB.isSelected();
			exportTiersRB = ExportTiersDialog.this.exportTiersRB.isSelected();
			exportLingTypesRB = ExportTiersDialog.this.exportLingTypesRB.isSelected();
			
			//save given directories so these can't change during export
			localDirectoryTextField = ExportTiersDialog.this.localDirectoryTextField.getText();
			sameDirectoryTextField = ExportTiersDialog.this.sameDirectoryTextField.getText();
			baseNameTextField = ExportTiersDialog.this.baseNameTextField.getText();
			suffixNameTextField = ExportTiersDialog.this.suffixNameTextField.getText();
		}
		
		/**
		 * Returns the directory where the corresponding file needs to be saved in.
		 * This depends on the selected options for the export
		 * @param file particular file where the directory is computed for
		 * @return the directory where the file needs to be saved
		 */
		private String getDirectoryToSave( File file ){
			String originalDirectory = file.getPath().substring( 0, file.getPath().length() - file.getName().length());
			
			//if original directory is selected
			if( originalDirRB ){
				return originalDirectory;
			}
			
			//if new directory has been selected
			if( newDirectoryRB )
				return originalDirectory + localDirectoryTextField;
			
			//if same directory has been chosen
			if( togetherInSameDirRB )
				return sameDirectoryTextField;
			
			return null;
		}
		
		/**
		 * Gets the file name of the specified transcription and edits it to the format specified for exportation
		 * @param transImpl the transcription implementation of the file
		 * @param nr the number of the saved file.
		 * 		  If all files are saved, the first file has number 0 and each file that follows is incremented by 1
		 * @return the name of the file to be saved or null if no name could be found
		 */
		private String getFileName( TranscriptionImpl transImpl, int nr ){
			
			if( originalFileNameRB )
				return transImpl.getName();
			
			if( originalFileNameWithSuffixRB ){
				String fileName = transImpl.getName();
				int index = fileName.lastIndexOf('.');
				return fileName.substring(0, index) + suffixNameTextField + fileName.substring(index);
			}
			
			if( useBaseNameWithSuffixRB ){
				String fileName = transImpl.getName();
				int index = fileName.lastIndexOf('.');
				return baseNameTextField + (nr+1) + fileName.substring(index);
			}
			
			return null;
		}
		
		/**
		 * Checks if the given directory exists. If it does not exist, it will
		 * try to create one.
		 * @param directoryPath the path to an existing (or nonexisting) directory
		 * @return boolean value indicating if directory exists (and is created succesfully if needed)
		 */
		private boolean createDirectory( String directoryPath ){
			boolean directoryExists = true;
			File directory = new File(directoryPath);
			
			//if directory does not exist, try to make it
			if( !directory.exists() )
				directoryExists = (new File(directoryPath)).mkdir();
			
			return directoryExists;
		}
		
		/**
		 * Set the relative paths for the media files
		 * @param pathName pathname where eaf file is saved too
		 * @param tr the transcription implementation that is going to be saved
		 */
		private void setRelativePaths( String pathName, TranscriptionImpl tr ){
			// update relative media paths
            // make sure the eaf path is treated the same way as media files,
            // i.e. it starts with file:/// or file://
			Vector linkedFiles = tr.getLinkedFileDescriptors();
			
            String fullEAFURL = FileUtility.pathToURLString(pathName);

            Vector mediaDescriptors = tr.getMediaDescriptors();
            MediaDescriptor md;
            String relUrl;

            for (int i = 0; i < mediaDescriptors.size(); i++) {
                md = (MediaDescriptor) mediaDescriptors.elementAt(i);
                relUrl = FileUtility.getRelativePath(fullEAFURL, md.mediaURL);
                md.relativeMediaURL = "file:/" + relUrl;
            }

            // linked other files 
            if (linkedFiles.size() > 0) {
                LinkedFileDescriptor lfd;

                for (int i = 0; i < linkedFiles.size(); i++) {
                    lfd = (LinkedFileDescriptor) linkedFiles.get(i);
                    relUrl = FileUtility.getRelativePath(fullEAFURL,
                            lfd.linkURL);
                    lfd.relativeLinkURL = "file:/" + relUrl;
                }
            }
		}
		
		/**
		 * Removes tiers from transcription based on the selected tiers in the export dialog 
		 * @param transImpl the transcription of which the tiers could be removed.
		 */
		private void removeTiersFromTranscription( TranscriptionImpl transImpl ){
			//get tiers in new set, so tiers can be removed from transImpl
			Vector tiers = new Vector( transImpl.getTiers() );
			
			//for each tier in transcription, check if it exists in the 
			//list of tiers that needs to be saved
			//if not, then remove it from the transcription
			for( int i=0; i<tiers.size(); i++ ){
				TierImpl tier = (TierImpl) tiers.get(i);
			
				//if tier is not selected for export, then remove its dependent tiers too
				if( !exists( tier.getName(), tiersToSave ) ){ 
					Vector dependentTiers = tier.getDependentTiers();
					
					//if dependent tiers should be neglected, then remove them all
					if( neglectDependentTiersRB ){						
						for( int n=0; n<dependentTiers.size(); n++ )
							transImpl.removeTier( (TierImpl)dependentTiers.get(n) );
					
						//remove parent tier
						transImpl.removeTier( tier );
					}else{
						//if a dependent tier exists that needs to be exported, don't remove the parent tier
						boolean removeParentTier = true;
						
						for( int n=0; n<dependentTiers.size(); n++ )
							if( exists( ((TierImpl)dependentTiers.get(n)).getName(), tiersToSave ) ){
								removeParentTier = false;
								break;
							}
						
						if( removeParentTier )
							transImpl.removeTier( tier );
					}
				}
			}
		}
		
		/**
		 * Removes tiers from transcription based on the selected linguistic types 
		 * in the export dialog 
		 * @param transImpl the transcription of which the tiers could be removed.
		 */
		private void removeTiersBasedOnLinguisticTypes( TranscriptionImpl transImpl ){
			//get tiers in new set, so tiers can be removed from transImpl
			Vector tiers = new Vector( transImpl.getTiers() );
			
			//for each tier in transcription, check if it exists in the 
			//list of tiers that needs to be saved
			//if not, then remove it from the transcription
			for( int i=0; i<tiers.size(); i++ ){
				TierImpl tier = (TierImpl) tiers.get(i);
				
				//if tier is not selected for export, then remove its dependent tiers too
				if( !exists( tier.getLinguisticType().getLinguisticTypeName(), tiersToSave ) ){ 
					Vector dependentTiers = tier.getDependentTiers();
					
					//if dependent tiers should be neglected, then remove them all
					if( neglectDependentTiersRB ){						
						for( int n=0; n<dependentTiers.size(); n++ )
							transImpl.removeTier( (TierImpl)dependentTiers.get(n) );
					
						//remove parent tier
						transImpl.removeTier( tier );
					}else{
						//if a dependent tier exists that needs to be exported, don't remove the parent tier
						boolean removeParentTier = true;
						
						for( int n=0; n<dependentTiers.size(); n++ )
							if( exists( ((TierImpl)dependentTiers.get(n)).getLinguisticType().getLinguisticTypeName(), tiersToSave ) ){
								removeParentTier = false;
								break;
							}
						
						if( removeParentTier )
							transImpl.removeTier( tier );
					}
				}
			}
		}
		
		/**
		 * The start of the save thread. All files will be saved here.
		 */
		public void run(){
			int failedExports = 0; //counter to count number of failed exports
			int refusedExports = 0; //nr exports refused because user indicated that files should not overwrite existing files
			int emptyFiles = 0; //nr of files not exported because they were empty
			boolean saveForever = false, skipForever = false;
			
			//walk through all transcriptions
			for( int f=0; f<transImplList.size(); f++ ){
				TranscriptionImpl transImpl = transImplList.get(f);

				if( exportTiersRB )
					removeTiersFromTranscription(transImpl);
				else if( exportLingTypesRB )
					removeTiersBasedOnLinguisticTypes(transImpl);
				else{
					LOG.warning("Warning: No export option chosen. Export won't have any effect");
					return;
				}
				
				//Try to save the transcription
				String path = getDirectoryToSave( eafFiles[f] );
				String fileName = getFileName( transImpl, f );
				if (path.charAt(path.length() - 1) != File.separatorChar) {
					path += File.separatorChar;
				}
				String directoryToSave = path + fileName;
				
				//save files
				try{
					//if directory does not exist, then create it
					boolean directoryExists = createDirectory(path);
					boolean saveThisFile = true;
					boolean fileExists = new File(directoryToSave).exists();
					
					if( fileExists && skipForever )
						saveThisFile = false;
					
					//overwrite files is not selected, then check if file exists and ask for overwriting
					if( !skipForever && !saveForever  && fileExists ){
						//show dialog to ask if existing file should be overwritten
						Object[] possibleValues = { "Yes To All", "Yes", "No", "No To All" };
						
						int choice = JOptionPane.showOptionDialog( null, 
								ElanLocale.getString("ExportTiersDialog.Message.OverwriteMessage.Description1") + " " + eafFiles[f].getName() + " " + ElanLocale.getString("ExportTiersDialog.Message.OverwriteMessage.Description2") + "\n\n" +
								ElanLocale.getString("ExportTiersDialog.Message.OverwriteMessage.Description3") + "\t " + eafFiles[f].getAbsolutePath() + "\n\n" +
								ElanLocale.getString("ExportTiersDialog.Message.OverwriteMessage.Description4") + "\n", ElanLocale.getString("ExportTiersDialog.Message.OverwriteMessage.Title"), 
										JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, possibleValues, possibleValues[2]);
						
						switch( choice ){
							case YES:
								saveThisFile = true;
								break;
							
							case YES_TO_ALL:
								saveForever = true;
								break;
								
							case NO_TO_ALL:
								skipForever = true;
								saveThisFile = false;
								break;
								
							default: //NO and other
								saveThisFile = false;
						}
							
					}
					
					//set relative paths
					setRelativePaths( directoryToSave, transImpl );
					
					//save files
					if( directoryExists )
						//if all files need to be saved OR this file need to be saved
						if( saveForever || saveThisFile ){
							if( !dontExportFilesWithoutTiersCB )
								ACMTranscriptionStore.getCurrentTranscriptionStore().storeTranscriptionIn(transImpl, null, new Vector(), directoryToSave, TranscriptionStore.EAF);
							else{
								if( transImpl.getTiers().size() > 0 )
									ACMTranscriptionStore.getCurrentTranscriptionStore().storeTranscriptionIn(transImpl, null, new Vector(), directoryToSave, TranscriptionStore.EAF);
								else
									emptyFiles++;
							}
						}else{
							refusedExports++;
						}
					else
						failedExports++;
				}catch(IOException e){
					LOG.warning("Can not write transcription to file with directory/filename: " + eafFiles[f].getAbsolutePath() + "/" + eafFiles[f].getName() );
					failedExports++;
				}
				
				//update progress bar
				progressValue++;
				progressBar.setValue(progressValue);
				progressBar.setString( Math.round(100 * (f+1)/(float)eafFiles.length) + "%" );
			}
			
			//exporting finished, so update progress bar
			progressBar.setString( ElanLocale.getString("ExportTiersDialog.ProgressBar.ExportingDone") );
			
			//show information on the export process
			String msg = (eafFiles.length-failedExports-refusedExports-emptyFiles) + " " + ElanLocale.getString("ExportTiersDialog.Message.OutOf") + " " + eafFiles.length + " " + ElanLocale.getString("ExportTiersDialog.Message.InfoMsg1");
			
			//construct additional details if not all files are exported succesfully
			if( emptyFiles + refusedExports + failedExports > 0 ){
				if( emptyFiles == 1 )
					msg += "\n\n" + emptyFiles + " " + ElanLocale.getString("ExportTiersDialog.Message.InfoMsg2single");
				else
					msg += "\n\n" + emptyFiles + " " + ElanLocale.getString("ExportTiersDialog.Message.InfoMsg2");
				
				if( refusedExports == 1 )
					msg += "\n" + refusedExports + " " + ElanLocale.getString("ExportTiersDialog.Message.InfoMsg3single");
				else
					msg += "\n" + refusedExports + " " + ElanLocale.getString("ExportTiersDialog.Message.InfoMsg3");
				
				if( failedExports == 1 )
					msg += "\n" + failedExports + " " + ElanLocale.getString("ExportTiersDialog.Message.InfoMsg4single");
				else
					msg += "\n" + failedExports + " " + ElanLocale.getString("ExportTiersDialog.Message.InfoMsg4");
			}
			
			if( failedExports > 0 )
				JOptionPane.showMessageDialog(ExportTiersDialog.this, msg, ElanLocale.getString("ExportTiersDialog.Message.ExportingDoneTitle"), JOptionPane.WARNING_MESSAGE);
			else
				JOptionPane.showMessageDialog(ExportTiersDialog.this, msg, ElanLocale.getString("ExportTiersDialog.Message.ExportingDoneTitle"), JOptionPane.INFORMATION_MESSAGE);
			
			//close dialog
			setVisible(false);
			dispose();
		}
	}
	
	/**
	 * Thread used to open files and shows the progress in a table
	 * @author Jeffrey Lemein
	 * @version March 2010
	 */
	private class OpenFilesThread extends Thread{
		/**
		 * Opens the files and stores relevant information for the export dialog/process
		 */
		public void run(){
			//show information in table corresponding to the opening process
			String txt = String.format( ElanLocale.getString("ExportTiersDialog.Message.LoadingTiers"), 0.0f, "%" );
			tierTable.setModel( new TierTableModel( new String[]{ txt, ElanLocale.getString("ExportTiersDialog.Message.Loading")} ) );
			
			txt = String.format( ElanLocale.getString("ExportTiersDialog.Message.LoadingLinguisticTypes"), 0.0f, "%" );
			linguisticTypeTable.setModel( new TierTableModel( new String[]{ txt, ElanLocale.getString("ExportTiersDialog.Message.Loading")} ) );
			
			//create list of transcriptionImpl that will hold the opened files and tierNames the tiers from those files
			transImplList = new ArrayList<TranscriptionImpl>();
			ArrayList<String> tierNames = new ArrayList<String>();
			ArrayList<String> linguisticTypes = new ArrayList<String>();
			
			//store all opened files as transcripionImpl
			for( int i=0; i<eafFiles.length; i++ ){
				if (eafFiles[i] == null) {
					continue;
				}
				transImplList.add( new TranscriptionImpl( eafFiles[i].getAbsolutePath() ) );
				Vector<TierImpl> listOfTiers = transImplList.get(i).getTiers();
				
				//add the tiers from the file to the list
				for( int j=0; j<listOfTiers.size(); j++ ){
					tierNames.add( listOfTiers.get(j).getName() );
					linguisticTypes.add( listOfTiers.get(j).getLinguisticType().getLinguisticTypeName() );
				}
				
				//don't update every time, but every 10 opened files
				if( i%10 == 0 ){
					txt = String.format( ElanLocale.getString("ExportTiersDialog.Message.LoadingTiers"), (100f*i)/eafFiles.length, "%" );
					tierTable.setValueAt(txt, 0, 0);
					txt = String.format( ElanLocale.getString("ExportTiersDialog.Message.LoadingLinguisticTypes"), (100f*i)/eafFiles.length, "%" );
					linguisticTypeTable.setValueAt(txt, 0, 0);
					
					tierTable.repaint();
					linguisticTypeTable.repaint();
				}
			}

			//sort the lists
			Collections.sort(tierNames);
			Collections.sort(linguisticTypes);
			
			//remove duplicates and update the table to contain selectable tiers
			removeDuplicates(tierNames);
			removeDuplicates(linguisticTypes);
			
			updateTierList(tierNames, linguisticTypes);
		}
	}

	
	/**
	 * TierTableModel is used for showing labels or a list of selectable tiers
	 * @author Jeffrey Lemein
	 * @version March 2010
	 */
	private class TierTableModel extends AbstractTableModel{
		private String[] columnNames;
		private Object[][] rowData;

		/**
		 * Create a table model with a column of checkboxes and a column of tier names
		 * @param tierNameList the tier names that needs to be showed
		 */
		public TierTableModel( List<String> tierNameList ){
			columnNames = new String[2];
	    	
	    	rowData = new Object[tierNameList.size()][2];
	    	
	    	for( int i=0; i<tierNameList.size(); i++){
	    		rowData[i][0] = false;
	    		rowData[i][1] = new String(tierNameList.get(i));
	    	}
		}
		
		/**
		 * Create a new table model with a list of labels that are shown vertically
		 * with each label in one row
		 * @param labels the list of labels that need to be showed
		 */
		public TierTableModel( String[] labels ){
			columnNames = new String[]{ "Information" };
			rowData = new String[labels.length][1];
			
			for( int i=0; i<labels.length; i++ )
				rowData[i][0] = labels[i];
		}
		
		/**
		 * Construct a default tier table model with a default message
		 */
	    public TierTableModel(){
	    	//create a table with just one cell with a default text
	    	columnNames = new String[1];
	    	
	    	rowData = new Object[1][1];
	    	rowData[0][0] = ElanLocale.getString("ExportTiersDialog.TableModel.DefaultMessage");
	    }

		public int getColumnCount() {
			return columnNames.length;
		}

		public int getRowCount() {
			return rowData.length;
		}
		
		public void setValueAt( Object object, int row, int column ){
			if( column == 0 )
				rowData[row][column] = object;
		}
		
		public boolean isCellEditable(int row, int col) {
	        //only the first column is editable (because it contains the checkboxes)
			return col <= 0;
	    }
		
		public Class getColumnClass(int c) {
			return getValueAt(0, c).getClass();
        }

		public Object getValueAt(int rowIndex, int columnIndex) {
			return rowData[rowIndex][columnIndex];
		}
		
		/**
		 * retrieves the selected tiers from the tier table list
		 * @return a list with the names of the selected tiers
		 */
		public List<String> getSelectedTiers(){
			List<String> tierNameList = new ArrayList<String>();
			
			if( columnNames.length == 2 ){
				for( int i=0; i<rowData.length; i++ )
					if( (Boolean)rowData[i][0] )
						tierNameList.add( (String)rowData[i][1] );
			}
			
			return tierNameList;
		}
	}
	
	
	
}
