package mpi.eudico.client.annotator.prefs.gui;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;

import mpi.eudico.client.annotator.prefs.PreferenceEditor;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.util.HashMap;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;


/**
 * A panel for viewer related preferences.
 * 
 * @author Han Sloetjes
 * @version 1.0
  */
public class ViewerPanel extends JPanel implements PreferenceEditor {
    private int origNumSubtitles = 4;
    private boolean origActiveAnnBold = false;
    private boolean origReducedTierHeight = false;
    
    private JComboBox numSubCB;
    private JCheckBox aaBoldCB;
    private JCheckBox redTierHeightCB;

    /**
     * Creates a new ViewerPanel instance
     */
    public ViewerPanel() {
        super();
        readPrefs();
        initComponents();
    }

    /**
     * Reads stored preferences.
     *
     */
    private void readPrefs() {
        Object val = Preferences.get("NumberOfSubtitleViewers", null);

        if (val instanceof Integer) {
            origNumSubtitles = ((Integer) val).intValue();
        }
        
        val = Preferences.get("TimeLineViewer.ActiveAnnotationBold", null);
        if (val instanceof Boolean) {
        	origActiveAnnBold = ((Boolean) val).booleanValue();
        }
        
        val = Preferences.get("TimeLineViewer.ReducedTierHeight", null);
        if (val instanceof Boolean) {
        	origReducedTierHeight = ((Boolean) val).booleanValue();
        }
    }

    private void initComponents() {
        setLayout(new GridBagLayout());

        Insets insets = new Insets(2, 0, 2, 0);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0;
        gbc.gridwidth = 2;
        gbc.insets = insets;
        add(new JLabel(ElanLocale.getString("Tab.Subtitles")), gbc);

        JLabel numLabel = new JLabel(ElanLocale.getString(
                    "PreferencesDialog.Viewer.NumSubtitles"));
        numLabel.setFont(numLabel.getFont().deriveFont(Font.PLAIN));
        gbc.gridy = 1;
        add(numLabel, gbc);

        numSubCB = new JComboBox(new Integer[] { 1, 2, 3, 4, 5, 6, 7, 8 });
        numSubCB.setSelectedItem(origNumSubtitles);
        gbc.gridy = 2;
        add(numSubCB, gbc);
        
        gbc.gridy = 3;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(12, 0, 2, 0);
        add(new JLabel(ElanLocale.getString("TimeLineViewer.Name")), gbc);
        
        aaBoldCB = new JCheckBox(ElanLocale.getString("TimeLineViewer.ActiveAnnotationBold"));
        aaBoldCB.setFont(aaBoldCB.getFont().deriveFont(Font.PLAIN));
        aaBoldCB.setSelected(origActiveAnnBold);
        gbc.gridy = 4;
        gbc.insets = insets;
        add(aaBoldCB, gbc);
        
        redTierHeightCB = new JCheckBox(ElanLocale.getString("TimeLineViewer.ReducedTierHeight"));
        redTierHeightCB.setFont(redTierHeightCB.getFont().deriveFont(Font.PLAIN));
        redTierHeightCB.setSelected(origReducedTierHeight);
        gbc.gridy = 5;
        add(redTierHeightCB, gbc);
        
        gbc.gridy = 10;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weighty = 1.0;
        add(new JPanel(), gbc); // filler
    }

    /**
     * Returns a map of changed key-value pairs.
     *
     * @return a map of changed key-value pairs
     */
    public Map getChangedPreferences() {
        if (isChanged()) {
            Map<String, Object> chMap = new HashMap<String, Object>(1);

            if (origNumSubtitles != (Integer) numSubCB.getSelectedItem()) {
                chMap.put("NumberOfSubtitleViewers",
                    (Integer) numSubCB.getSelectedItem());
            }
            
            if (origActiveAnnBold != aaBoldCB.isSelected()) {
            	chMap.put("TimeLineViewer.ActiveAnnotationBold", new Boolean(aaBoldCB.isSelected()));
            }
            
            if (origReducedTierHeight != redTierHeightCB.isSelected()) {
            	chMap.put("TimeLineViewer.ReducedTierHeight", new Boolean(redTierHeightCB.isSelected()));
            }
            
            return chMap;
        }

        return null;
    }

    /**
     * Returns whether any preference item has been changed.
     *
     * @return true if anything has changed.
     */
    public boolean isChanged() {
        int count = (Integer) numSubCB.getSelectedItem();

        if (count != origNumSubtitles) {
            return true;
        }
        
        if (origActiveAnnBold != aaBoldCB.isSelected()) {
        	return true;
        }
        
        if (origReducedTierHeight != redTierHeightCB.isSelected()) {
        	return true;
        }
        
        return false;
    }
}
