/*
 * Created on Mar 30, 2004
 * $Id: LogUtil.java 8222 2007-02-22 10:20:51Z klasal $
 */
package mpi.util;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;

import java.util.logging.LogManager;


/**
 * Convenience methods for logging configuration and formatting.
 *
 * @author Wouter Huijnink
 */
public final class LogUtil {
    /**
     * Creates a new LogUtil instance. Only present to allow of configuring the
     * logging mechanism:
     * 
     * <p>
     * If the "java.util.logging.config.class" property is set, then the
     * property value is treated as a class name. The given class will be
     * loaded, an object will be instantiated, and that object's constructor
     * is responsible for reading in the initial configuration. (That object
     * may use other system properties to control its configuration.) The
     * alternate configuration class can use readConfiguration(InputStream) to
     * define properties in the LogManager. (<code>LogManager</code>
     * javadocs).
     * </p>
     * 
     * <p>
     * Usage: <code>java
     * -Djava.util.logging.config.class=mpi.library.util.LogUtil
     * -Dlogging.config.file=LOGGING_CONFIG_FILE</code>
     * </p>
     */
    public LogUtil() {
        try {
            System.out.println("Initializing logging configuration...");

            String configFile = System.getProperty("logging.config.file");

            if (configFile != null) {
                LogManager.getLogManager().readConfiguration(ClassLoader.getSystemResourceAsStream(
                        configFile));
            } else {
                System.err.println(
                    "Logging could not be configured (system property logging.config.file not found) - logging will not be available.");
            }
        } catch (SecurityException e1) {
            System.err.println(
                "Logging could not be configured - logging will not be available.");
            e1.printStackTrace(System.err);
        } catch (IOException e1) {
            System.err.println(
                "Logging could not be configured - logging will not be available.");
            e1.printStackTrace(System.err);
        }
    }

    /**
     * Returns a String representation of a Throwable's stack trace.
     *
     * @param t The Throwable.
     *
     * @return A String representation of the Throwable's stack trace.
     */
    public static String formatStackTrace(Throwable t) {
        StringWriter sw = new StringWriter();
        t.printStackTrace(new PrintWriter(sw));

        return sw.toString();
    }
}
